 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.4 $
 *
 * $Id: ClientCommunication.cpp,v 1.4 2001/04/11 23:29:48 daville Exp $
 * $Log: ClientCommunication.cpp,v $
 * Revision 1.4  2001/04/11 23:29:48  daville
 * Optimisation de la ngociation d'une connexion au serveur.
 *
 * Revision 1.3  2001/03/18 22:26:35  daville
 * Intgration du fichier ClientToolsXMProtocol.h et ajout de lignes fin fichier.
 *
 * Revision 1.2  2000/11/02 21:43:49  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


#include <ClientCommunication.h>
#include <ClientToolsXMProtocol.h>

extern void GThreadWait(double );

static  double gThreadWait=0.1;
static	int gIDListener=0;
static	int gStatusListener=0;

int SendTo(int idWrite, XMQuery *pQuery);
XMReponse * ReceiveFrom(int idRead);
SCConnection *MakeSCConnection(XMPipe *pipe, int id_client);

void SetThreadWait(double pause)
{
	gThreadWait = pause;
}

SCConnection *MakeSCConnection(XMPipe *pipe, int id_client)
{
	SCConnection *con;
	con = new SCConnection;
	con->mID = id_client;
	con->mPipe = pipe;
	return con;
}

XMPipe *CreerFluxCommunication(char *canal, int id, XMPipe *pipe)
{
	char AdresseW[strlen(canal)+strlen("/XMCorePipe")+11];
	char AdresseR[strlen(canal)+strlen("/XMCorePipe")+11];
	int RStatut, WStatut;
	
	if (id != -1)
	{
		sprintf(AdresseW, "%s/XMCorePipe%5.5dW", canal, id);
		sprintf(AdresseR, "%s/XMCorePipe%5.5dR", canal, id);
	}else
	{
		sprintf(AdresseW, "%s/XMCorePipeW", canal);
		sprintf(AdresseR, "%s/XMCorePipeR", canal);
	}
	if ((pipe->IDWrite = open(AdresseW, O_RDWR)) == -1)
	{
		printf("Erreur d'ouverture du canal de communication [%s]\n", AdresseW);
		delete pipe;
		return NULL;
	}
	if ((pipe->IDRead = open(AdresseR, O_RDWR)) == -1)
	{
		printf("Erreur d'ouverture du canal de communication [%s]\n", AdresseR);
		delete pipe;
		return NULL;
	}
	WStatut = fcntl(pipe->IDWrite, F_GETFL);
	fcntl(pipe->IDWrite, F_SETFL, WStatut | O_NONBLOCK);
	RStatut = fcntl(pipe->IDRead, F_GETFL);
	fcntl(pipe->IDRead, F_SETFL, RStatut | O_NONBLOCK);
	return pipe;
}

SCConnection *ConnectToServer(char *adresse)
{
	SCConnection *con=NULL;
	XMPipe *pipe;
	XMReponse *reponse;
	
	pipe = CreerFluxCommunication(adresse, -1, new XMPipe);
	if (!pipe) return NULL;
	SendTo(pipe->IDWrite, MakeQuery(new XMQuery, -1, CQCONNECTION, NULL));
	for (int k=0;(!(reponse = ReceiveFrom(pipe->IDRead))) && (k<NB_TENTATIVE_RECEIVE); k++) sleep(1);
	close(pipe->IDRead); close(pipe->IDWrite);
	delete pipe;
	if (reponse)
		return MakeSCConnection(CreerFluxCommunication(adresse, reponse->IDClient, new XMPipe), reponse->IDClient);
	return NULL;
}

int SendTo(int idWrite, XMQuery *pQuery)
{
	int Result = -1;
	for (int k=0; ((Result = write(idWrite, pQuery, sizeof(XMQuery)) == -1) && (k<NB_TENTATIVE_SEND)); k++);
	return Result;
}

int SendToServer(SCConnection * con, XMQuery *pQuery)
{
	int Result = -1;
	for (int k=0; ((Result = write(con->mPipe->IDWrite, pQuery, sizeof(XMQuery)) == -1) && (k<NB_TENTATIVE_SEND)); k++);
	return Result;
}

XMReponse * ReceiveFrom(int idRead)
{
	static XMReponse Flux;
	return (read(idRead, &Flux, sizeof(XMReponse)) ==-1?NULL:&Flux);
}

XMReponse * ReceiveFromServer(SCConnection * con)
{
	int Result = -1;
	static XMReponse Flux;
	for (int k=0; ((Result=read(con->mPipe->IDRead, &Flux, sizeof(XMReponse)))==-1) && (k<NB_TENTATIVE_RECEIVE); k++);
	return (Result == -1? NULL: &Flux);
}

int DisconnectToServer(SCConnection * con)
{
	int Result=0;
	XMQuery query;
	
	/* Not connected */
	if (!con) return 2;
	if ((Result=SendToServer(con, MakeQuery(&query, con->mID, CQDECONNECTION, NULL))) ==-1) return 0;
	if ((Result=(int)ReceiveFromServer(con)) ==0) return 0;
	return 1;
}

int ShutdownServer(SCConnection * con)
{
	int Result=0;
	XMQuery query;
	
	/* Not connected */
	if (!con) return 2;
	if ((Result=SendToServer(con, MakeQuery(&query, con->mID, CQSHUTDOWN, NULL))) ==-1) return 0;
	if ((Result=(int)ReceiveFromServer(con)) ==0) return 0;
	return 1;
}

/*************************************************/
/*************************************************/
XMQuery *InitQuery(XMQuery *query)
{
	if (!query) return NULL;
	query->IDQuery = CQUNKNOWN;
	return query;
}
/*************************************************/
/*************************************************/

XMQuery * MakeQuery(XMQuery *query, int id_client, int id_query, char *data)
{
	int len,k;
	query->IDClient	= id_client;
	query->IDQuery = id_query;
	if (!data) sprintf(query->Query, "%d#", id_query); else sprintf(query->Query, "%d#%s#", id_query, data);
	return query;
}

/*************************************************/
/*************************************************/
int 	 ReadReponse(XMReponse *reponse, int &id_client, int &id_reponse, char *data)
{
	id_client = reponse->IDClient;
	id_reponse = reponse->IDReponse;
	data = new char[strlen(reponse->Reponse) + 1];
	sprintf(data, "%s", reponse->Reponse);
	printf("IDClient %d   IDReponse %d   Data %s\n", id_client, id_reponse, data);
	return id_reponse;
}
/*************************************************/
/*************************************************/


typedef struct Pack
{
	int IDRead;
	HandleServer *Hdle;
};

void *ListenerServer(void *data)
{
	XMReponse *reponse;
	int IDRead = (int)(((Pack *)data)->IDRead);
	HandleServer *Hdle = (HandleServer *)(((Pack *)data)->Hdle);
	int k;
	
	while (gStatusListener)
	{
		//select(...)
		if ((reponse = ReceiveFrom(IDRead)) != NULL)
			Hdle->TreateReponse(reponse);
		GThreadWait(gThreadWait);
	}
	printf("Fin client  l'coute\n");
	return NULL;
}

/********************************************************************/
int ListenToServer(SCConnection * con, int status, HandleServer *hdle)
{
	//printf("Appel listener: IDUSER %d    STATUS %d    Hdle: %p\n", con->mID, status, hdle);
	if (!status) gStatusListener=0;
	//if (gStatusListener) return gStatusListener;
	
	pthread_t thread;
	pthread_attr_t pthread_attr;
	Pack *pack;
	
	pack = new Pack;
	//printf("Listener sur: IDREAD %d   Hdle: %p\n", con->mPipe->IDRead, hdle);
	pack->IDRead = con->mPipe->IDRead;
	pack->Hdle = hdle;
	pthread_attr_init(&pthread_attr);
	pthread_create (&thread, &pthread_attr, ListenerServer, pack);
	
	return (gStatusListener=thread);
}


