 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.3 $
 *
 * $Id: Liste.h,v 1.3 2001/04/03 23:12:27 daville Exp $
 * $Log: Liste.h,v $
 * Revision 1.3  2001/04/03 23:12:27  daville
 * Ajout d'un cast pour l'appel  la fonction delete.
 * Ajout de return pour les fonctions qui retournent des donnes.
 *
 * Revision 1.2  2000/11/02 22:10:22  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


#ifndef LIST
#define LIST
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "number.h"


typedef struct SCell
{
	void * ptrInfo;
	struct SCell * ptrNext;
	struct SCell * ptrPrev;
} SCellule;

template <class T> class CList
{
	SCellule * mFirst;
	SCellule * mCurrent;
	SCellule * mLast;
	int mCount;
	public :
	CList();
	CList(CList <T> &);
	~CList();

	// On ajoute un element en fin de liste
	void Add(T );
	void AddAndGo(T );
	
	// On insere un element avant la cellule courante ea
	// La cellule contenant cet element devient la cellule courante
	void Ins(T );

	// On remplace l'element de la cellule courante
	void Mod(T );

	// On supprime la cellule courante et on met dans 
	// le parametre l'element de la cellule supprimee
	// La nouvelle cellule courante est la suivante si elle existe
	// la precedente sinon
	void Del(T &);

	// La cellule courante devient la premiere cellule et on met 
	// dans le parametre l'element contenu dans cette cellule
	void MoveFirst(T &);

	// La cellule courante devient la cellule suivante
	// Si la nouvelle cellule courante existe, on met dans le
	// parametre l'element contenu dans cette cellule
	void MoveNext(T &);

	// Si la cellule precedent la cellule courante existe
	// elle devient la cellule courante
	// Si la nouvelle cellule courante existe, on met dans le
	// parametre l'element contenu dans cette cellule
	void MovePrev(T &);

	// La cellule courante devient la derniere cellule
	// Si la nouvelle cellule courante existe, on met dans le
	// parametre l'element contenu dans cette cellule
	void MoveLast(T &);
	void Init();
	// On teste si la cellule courante n'existe pas
	int Eol();
	
	//Teste si la cellule suivante existe.
	int WellEol();

	// On renvoie l'element courant.
	void Refresh(T &Info);

	// Recherche le premiere element correspondant
	// Si 'Posionner' =0 alors la position courante
	// apres recherche sera conservee.
	int ListSearch(T Info, int Positionner=0);
	
	// Recherche l'element en argument 'Old', et remplace par l'argument 'New'
	// Renvoye 1 si l'operation s'est bien passee, 0 sinon
	int Replace(T Old, T New);	

	// On renvoie le nombre de cellules dans la liste
	int GetCount();
	
	// Surcharge de l'operateur =
	CList <T> & operator = (CList <T> &);
	T Get(int k);
	T GetID(int k);
};

template <class T> CList <T> :: CList(CList <T> & Liste)
{
	mFirst = mCurrent = mLast = NULL;
	*this = Liste;
}

template <class T> CList <T> & CList<T> :: operator = (CList <T> & Liste)
{
	SCellule * ptrCell, * Tmp;
	T * ptrT, TmpT;

	ptrCell = mFirst;
	while(ptrCell != NULL)
	{
		ptrT = (T *)(ptrCell->ptrInfo);
		delete ptrT;
		Tmp = ptrCell;
		ptrCell = ptrCell->ptrNext;
		delete Tmp;
	}

	Liste.MoveFirst(TmpT);
	mFirst = mCurrent = mLast = NULL;
	while( !Liste.Eol())
	{
		Add(TmpT);
		Liste.MoveNext(TmpT);
	}
	return *this;   /* pour les egalites en cascade */	
}
	
template <class T> void CList <T> :: Init()
{
	T Info;
	MoveFirst();
	while (!Eol()) Del(Info);
}
	

template <class T> CList <T> :: CList()
{
	mFirst = NULL;
	mCurrent = NULL;
	mLast = NULL;
	mCount = 0;
}

template <class T> void CList <T> :: Add( T   Info)
{
	T * ptrInfo = new T;
	SCellule * ptrCellule = new SCellule;

	assert( ptrCellule != NULL);
	assert( ptrInfo != NULL);
	*ptrInfo = Info;
	ptrCellule->ptrInfo = (void *)ptrInfo; 
	ptrCellule->ptrNext = NULL;
	ptrCellule->ptrPrev = mLast;
	if( mLast != NULL)
		mLast->ptrNext = ptrCellule;
	mLast = ptrCellule;
	if(mFirst == NULL)
	{
		mFirst = mCurrent = mLast;
	}
	mCount++;
}

template <class T> void CList <T> :: AddAndGo(T Info)
{
        Add(Info);
        mCurrent=mLast;
}

template <class T> void CList <T> :: MoveFirst( T & Info)
{
	mCurrent = mFirst;

	if(mCurrent != NULL)
		Info = *((T *)(mCurrent->ptrInfo));
}

template <class T> void CList <T> :: MoveNext( T & Info)
{
	if(mCurrent != NULL)
		mCurrent = mCurrent->ptrNext;
	if(mCurrent != NULL)
		Info = *((T *)(mCurrent->ptrInfo));
}

template <class T> int CList <T> :: Eol()
{
	return (mCurrent == NULL);
}

template <class T> int CList <T> :: WellEol()
{
        return (mCurrent->ptrNext==NULL);
}

template <class T> void CList <T> :: MovePrev( T & Info)
{
	if(mCurrent != mFirst)
		mCurrent = mCurrent->ptrPrev;
	if(mCurrent != NULL)
		Info = *((T *)(mCurrent->ptrInfo));
}

template <class T> void CList <T> :: MoveLast(T &Info)
{
        if (mLast) mCurrent=mLast;
        if (mLast) Info=*((T*)(mCurrent->ptrInfo));
}

template <class T> CList <T> :: ~CList()
{
	SCellule * Tmp;

	mCurrent = mFirst;
	while( mCurrent != NULL)
	{
		Tmp = mCurrent;
		mCurrent = mCurrent->ptrNext;
		delete (T *)(Tmp->ptrInfo);
		delete Tmp;
	}
}

template <class T> void CList <T> :: Del( T  & Info)
{
	if ( mCurrent == NULL)
		return;
	Info = *((T *)(mCurrent->ptrInfo));
	if(mCurrent->ptrPrev != NULL)
		mCurrent->ptrPrev->ptrNext = mCurrent->ptrNext;
	if(mCurrent->ptrNext != NULL)
		mCurrent->ptrNext->ptrPrev = mCurrent->ptrPrev;
	SCellule * Tmp = mCurrent;

	if(mCurrent->ptrNext != NULL)
		mCurrent = mCurrent->ptrNext;
	else
		mCurrent = mCurrent->ptrPrev;
	if( Tmp == mFirst)
		mFirst = mCurrent;
	if( Tmp == mLast)
		mLast = mCurrent;
	mCount--;
}

template <class T> void CList <T> :: Mod( T   Info)
{
	if ( mCurrent != NULL)
		*((T *)(mCurrent->ptrInfo)) = Info;
}

template <class T> void CList <T> :: Ins( T   Info)
{
	if ( mCurrent == NULL)
	{
		Add(Info);
		return;
	}
	SCellule * ptrCellule = new SCellule;

	assert(ptrCellule != NULL);
	ptrCellule->ptrNext = mCurrent;
	ptrCellule->ptrPrev = mCurrent->ptrPrev;
	if( mCurrent->ptrPrev != NULL)
		mCurrent->ptrPrev->ptrNext = ptrCellule;
	mCurrent->ptrPrev = ptrCellule;

	T * ptrInfo = new T;
	assert(ptrInfo != NULL);
	* ptrInfo = Info;

	ptrCellule->ptrInfo = (void *)ptrInfo;
	if( mCurrent == mFirst)
		mFirst = ptrCellule;
	mCurrent = ptrCellule;
	mCount++;
}

template <class T> void CList<T> ::Refresh(T &Info)
{
        Info=*((T*)(mCurrent->ptrInfo));
}

template <class T> int CList <T> ::ListSearch(T Info, int Positionner=0)
{
        T info;
        SCellule *Ptr=mCurrent;
        MoveFirst(info);
        while (!Eol())
        {
                if (info==Info) break;
                MoveNext(info);
        }
        if (!Positionner) mCurrent=Ptr;
        return (info==Info);
}

template <class T> int CList <T> :: Replace(T Old, T New)
{
        T  *Ptr;
        if (ListSearch(Old,1))
        {
                delete ((T *)(mCurrent->ptrInfo));
        	Ptr=new T;
        	*Ptr=New; 
        	mCurrent->ptrInfo=(void *)Ptr;
        	return 1;
        }
        return 0;
}

template <class T> int CList <T> :: GetCount()
{
	return mCount;
}

template <class T> T CList<T> :: Get(int k)
{
	int i=0;
	T t;
	for(MoveFirst(t); (!Eol()); MoveNext(t)) if (i++==k) break;
	return t;
}

template <class T> T CList<T> :: GetID(int k)
{
	int i=0;
	T t;
	for(MoveFirst(t); (!Eol()); MoveNext(t)) if (t->ID()==k) break;
	return t;
}

#endif
