 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.3 $
 *
 * $Id: calculator.h,v 1.3 2001/03/18 22:22:27 daville Exp $
 * $Log: calculator.h,v $
 * Revision 1.3  2001/03/18 22:22:27  daville
 * Ajout de lignes fin fichier.
 *
 * Revision 1.2  2000/11/02 21:42:11  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


#include <stdio.h>
#ifndef CCALCULATOR_H
#define CCALCULATOR_H


template <class T> class CCalculator
{
        private:
        	T* (*mfCalc)(OnState st, T *, T *);
        	char *mMsgError;
        	TObjectX mType;
        public:
        	CCalculator();
        	CCalculator(CCalculator &cal);
        	CCalculator(TObjectX type, T* (*fcalc)(OnState st, T *, T *), char *err=0);
        	void SetTObject(TObjectX type);
        	void SetMsgError(char *err);
        	void SetfCalc(T* (*fcalc)(OnState st, T *, T *));
        	T *Evaluate(OnState st, T *objg, T *objd);
        	TObjectX GetTypeObject();
        	~CCalculator();
};



template <class T> CCalculator<T>::CCalculator()
{
        mMsgError = NULL;
        mfCalc = NULL;
        mType = TO_UNKNOWN;
}

template <class T> CCalculator<T>::CCalculator(CCalculator &cal)
{
        mfCalc = cal.mfCalc;
        if (cal.mMsgError)
        {
                mMsgError = new char[strlen(cal.mMsgError)+1];
                strcpy(mMsgError, cal.mMsgError);
        } else mMsgError = NULL;
        mType = cal.mType;
}

template <class T> CCalculator<T>::CCalculator(TObjectX type, T* (*fcalc)(OnState st, T *, T *), char *err)
{
        mType = type;
        if (err)
        {
                mMsgError = new char[strlen(err)+1];
                strcpy(mMsgError, err);
        } else mMsgError = NULL;
        mfCalc = fcalc;
}

template <class T> void CCalculator<T>::SetTObject(TObjectX type)
{
        mType = type;
}

template <class T> void CCalculator<T>::SetMsgError(char *err)
{
        if (err)
        {
                mMsgError = new char[strlen(err)+1];
                strcpy(mMsgError, err);
        } else mMsgError = NULL;
}

template <class T> void CCalculator<T>::SetfCalc(T* (*fcalc)(OnState st, T *, T *))
{
        mfCalc = fcalc;
}

template <class T> T* CCalculator<T>::Evaluate(OnState st, T *objg, T *objd)
{
        T *po = ((*mfCalc)(st, objg, objd));
        po->Type() = mType;
        return po;
}

template <class T> TObjectX CCalculator<T>::GetTypeObject()
{
        return mType;
}

template <class T> CCalculator<T>::~CCalculator()
{
        mfCalc = NULL;
        if (mMsgError) delete mMsgError;
        mMsgError = NULL;
        mType = TO_UNKNOWN;
}

#endif  /*  CCALCULATOR_H  */
