 /***********************************************************************
 * =======================================				*
 * EasyMaths - Core component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.3 $
 *
 * $Id: polynome.cpp,v 1.3 2001/04/03 23:52:59 daville Exp $
 * $Log: polynome.cpp,v $
 * Revision 1.3  2001/04/03 23:52:59  daville
 * Ajout de lignes fin de fichier.
 *
 * Revision 1.2  2000/11/02 22:09:51  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


#include <string.h>
#include <stdlib.h>
#include "number.h"
#include "polynome.h"

template <class T> T Max(T x, T y)
{
        return ((x>y) ? x: y);
}

template <class T> T Min(T x, T y)
{
        return ((x>y) ? x: y);
}

Polynome::Polynome()
{
        Po=NULL;
        Deg=0;
}

Polynome::Polynome(int deg)
{
        if (deg<0) return;
        Deg=deg;
        Po=new SPol[Deg+1];
        int i;
        for (i=0;i<=Deg;i++) Po[i].Xs=NULL;
}

Polynome::Polynome(Polynome &P)
{
        if (!P.Po) return;
        SPol SP;
        int i;
        Deg=P.Deg;
        Po=new SPol[Deg+1];
        for(i=0;i<=Deg;i++)
        {
                if (P.Po[i].Xs)
                {
                        SP.Xs=new char[strlen(P.Po[i].Xs)+1];
                        strcpy(SP.Xs,P.Po[i].Xs);
                } else SP.Xs=NULL;
                SP.X=P.Po[i].X;
                Po[i]=SP;
        }
}

void Polynome::SetDeg(int D)
{
        if (D<0) return;
        Deg=D;
        Po=new SPol[Deg+1];
        int i;
        for (i=0;i<=Deg;i++) Po[i].Xs=NULL;
}

void Polynome::Set(int i, float x)
{
        if ((i>Deg) || (!Po)) return;
        char Vals[15];
        Po[i].X=x;
        gcvt(x,15,Vals);
        Po[i].Xs=new char[strlen(Vals)+1];
        strcpy(Po[i].Xs,Vals);
}

Polynome &Polynome::operator= (Polynome P)
{
        int i;
        if (Po)
        {
                for(i=0;i<=Deg;i++) if (Po[i].Xs) delete Po[i].Xs;
                delete Po;
        }
        Deg=P.Deg;
        Po=new SPol[Deg+1];
        for(i=0;i<=P.Deg;i++)
        {
                if (P.Po[i].Xs)
                {
                        Po[i].Xs=new char[strlen(P.Po[i].Xs)+1];
                	strcpy(Po[i].Xs, P.Po[i].Xs);
                } else Po[i].Xs=NULL;
                Po[i].X=P.Po[i].X;
        }
        return *this;
}                

Polynome Polynome::operator+ (Polynome &P)
{
        int i=Max(P.Deg,Deg);
        Polynome Q(i);
        SPol SP,TSP[2];
        SP.X=0;
        SP.Xs="0";
        TSP[0]=SP;
        TSP[1]=SP;
        for(i=0;(i<=Q.Deg);i++)
        {
                if ((i<=Deg) && (i<=P.Deg))
                {
                        if (Po)   {if (!Po[i].Xs)  ; else TSP[0]=Po[i];}
                        if (P.Po) {if (!P.Po[i].Xs); else TSP[1]=P.Po[i];}
                } else
                	if ((P.Po) && (i>Deg)) TSP[1]=P.Po[i];
                		else if (Po) TSP[0]=Po[i];
                Q.Po[i].Xs=Simplifie_Somme(TSP[0].Xs, TSP[1].Xs);
                Q.Po[i].X=TSP[0].X+TSP[1].X;
                TSP[0]=SP;
                TSP[1]=SP;
        }
        Q.Refresh();
        return Q;
}                

Polynome Polynome::operator- (Polynome &P)
{
        int i=Max(P.Deg,Deg);
        Polynome Q(i);
        SPol SP,TSP[2];
        SP.X=0;
        SP.Xs="0";
        for(i=0;(i<=Q.Deg);i++)
        {
                if ((i<=Deg) && (i<=P.Deg))
                {
                        if (!Po[i].Xs) ; else TSP[0]=Po[i];
                        if (!P.Po[i].Xs) ; else TSP[1]=P.Po[i];
                } else if (i>Deg) TSP[1]=P.Po[i]; else TSP[0]=Po[i];
                Q.Po[i].Xs=Simplifie_Soust(TSP[0].Xs, TSP[1].Xs);
                Q.Po[i].X=TSP[0].X-TSP[1].X;
                TSP[0]=SP;
                TSP[1]=SP;
        }
        Q.Refresh();
        return Q;
}                

Polynome Polynome::operator* (Polynome &P)
{
        int i,j;
        Polynome *P1,P2;
        char *TChar[2];
        for(i=0;i<=Deg;i++)
        {
                TChar[0]=TChar[1]="0";
                P1=new Polynome(i+P.Deg);
                for(j=0;j<=P.Deg;j++)
                {
                        P1->Po[j+i].X =(Po[i].X)*(P.Po[j].X);
                        if (Po[i].Xs)   TChar[0]=Po[i].Xs;
                        if (P.Po[j].Xs) TChar[1]=P.Po[j].Xs;
                        P1->Po[j+i].Xs=Simplifie_Multi(TChar[0],TChar[1]);
                }
                P2=(*P1)+P2;
                delete P1;
        }
        return P2;
}

void Polynome::Refresh()
{
        int i=Deg;
        while ((i>0) && (!Po[i].X) && (Po[i].Xs)) i--;
        if (i!=Deg)
        {
                SPol *Ptr;
                Deg=i;
                if (!i) i++;
                //Ptr=new SPol[i];
                //for (i=0;i<=Deg;i++) Ptr[i]=Po[i];
                //delete Po;
                //Po=Ptr;
        }
}

Polynome Reduction(Polynome P1, Polynome P2, Polynome &Reste)
{
        int p;
        p=P1.Degre()-P2.Degre();
        Polynome Quotient;
        if (p<0)
        {
                Reste=P1;
                Quotient.SetDeg(0);
                Quotient.Set(0,0);
                return Quotient;
        }
        Quotient.SetDeg(p);
        float S=((P1.Po[P1.Deg].X)/(P2.Po[P2.Deg].X));
        Quotient.Set(p,S);
        Polynome Dif,Prd,Res;
        Prd=Quotient*P2;
        Dif=P1-Prd;
        Dif.Refresh();
        Res=Reduction(Dif,P2,Reste);
        
        Quotient=Quotient+Res;
        Quotient.Refresh();
        return Quotient;
}

void Polynome::Lire()
{
	int i;
	if (Po)
	{
	       printf("\nP=");
	       for(i=Deg;i>=0;i--) printf("%f+ ",Po[i].X);
	       printf("\nP=");
	}
}

void Polynome::Lires()
{
	int i;
	if (Po)
	{
	       printf("\nP= %s*x^%d",Po[Deg].Xs,Deg);
	       for(i=Deg-1;i>=0;i--) printf("+ %s*x^%d",Po[i].Xs,i);
	       printf("\n");
	}
}
 
float Polynome::operator[](int i)
{
        return Po[i].X;
}       

Polynome::~Polynome()
{
        if (Po)
        {
                int i;
                for(i=0;i<=Deg;i++)
                {
                        if (Po) {if (Po[i].Xs)  delete Po[i].Xs;}
                }
                delete Po;Po=NULL;
        }
}


