 /***********************************************************************
 * =======================================				*
 * EasyMaths - IHM component of EasyMaths				*
 * November 2000							*
 * =======================================				*
 *									*
 * Copyright (C) 2000, 2001 Didier Davill				*
 * Developed by Didier Davill <Didier.Daville@ddarea.com>		*
 *									*
 * This library is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU Library General Public License as 	*
 * published by the Free Software Foundation; either version 2 of the	*
 * License, or (at your option) any later version.			*
 *									*
 * This library is distributed in the hope that it will be useful, but	*
 * WITHOUT ANY WARRANTY; without even the implied warranty of		*
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.			*
 * See the GNU Library General Public License for more details.		*
 *									*
 * You should have received a copy of the GNU Library General Public	*
 * License along with this library; if not, write to the Free Software	*
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA		*
 * 02111-1307, USA.							*
 ***********************************************************************/

 /***********************************************************************
 * $Author: daville $
 *
 * $Revision: 1.6 $
 *
 * $Id: easymaths.cc,v 1.6 2001/04/14 14:30:59 daville Exp $
 * $Log: easymaths.cc,v $
 * Revision 1.6  2001/04/14 14:30:59  daville
 * On va chercher les donnes d'initialisation dans le rpertoire $home/.Easymaths/.
 *
 * Revision 1.5  2001/04/04 20:13:55  daville
 * Ajout des directives d'inclusion du fichier libgen.h
 * Ajout de lignes fin de fichier.
 *
 * Revision 1.4  2000/11/11 15:06:58  daville
 * On ne fait appel  la fonction GThreadInit qu'une seule fois.
 *
 * Revision 1.3  2000/11/11 14:38:40  daville
 * Mise en commentaire de la fonction FermerFichierConf.
 * On fait appel  la fonction close.
 *
 * Revision 1.2  2000/11/02 22:29:12  daville
 * Intgration de l'entte de copyright et de suivi de version.
 *
 *
 ***********************************************************************/


/*
easymaths Application
Main unit implementation file:easymaths.cc
*/
#include <signal.h>
#include <easymaths.h>
#include <easymaths_signal.h>
#include <VGlobal.h>
#include <Core/xmgthread.h>
#include <string.h>
#include <FichierConf.h>
#include <libgen.h>

#define NOM_FICHIER_CONF		"EasyMaths.conf"

/*
main program
*/
int main (int argc, char *argv[])
{
	EasymathsApp app(&argc, argv);
        	app.Run();
	return 0;
}

// Easymaths MAIN FORM  CLASS
/*
main form constructor
*/
EasymathsForm::EasymathsForm(VDKApplication* app, char* title):
	VDKForm(app,title)
{
	mProgress = NULL;
	mCurrentFile[0] = '\0';
	IsRunning = false;
}

/*
main form destructor
*/
EasymathsForm::~EasymathsForm()
{
}

/*
main form setup
*/
void
EasymathsForm::Setup(void)
{

	FILE *pFile=NULL;
	char *Parametre=NULL;
	
	/* Initialize tread attributes */
	GThreadInit(NULL);
	
	/* Build the filename configuration */
	Parametre = new char[strlen(getenv("HOME"))+strlen("/.EasyMaths/")+strlen(NOM_FICHIER_CONF)+1];
	sprintf(Parametre, "%s/.EasyMaths/%s", getenv("HOME"), NOM_FICHIER_CONF);
	
	/* Try to open the file configuration */
	if (!(pFile = OuvrirFichierConf(Parametre)))
	{
		Application()->MessageBox("Erreur de chargement",
		"Le fichier de chargement de donnes de EasyMaths n'a pas t trouv!\n \
		Utiliser les outils du serveur pour dfinir les environnement d'un nouveau utilisateur.", MB_OK);
		delete Parametre;
	}
	else
	{
		delete Parametre;
		if (!(Parametre = LireParametreFichierConf(pFile, "FLUX", "FLUX_DIR")))
			Application()->MessageBox("Erreur de chargement",
				"Le canal de communication avec le server n'a pas\nt trouv dans le fichier d'initialisation.", MB_OK); else
		{
			mConnection = ConnectToServer(Parametre);
			if (mConnection != NULL) ListenToServer(mConnection, 1, this);
				else Application()->MessageBox("Erreur de connection", "Impossible de se connecter au server maintenant", MB_OK);
			delete Parametre;
		}
		FermerFichierConf(pFile);
	}
	GUISetup(); // vdkb gui setup
// put your code below here
}

// Easymaths APPLICATION  CLASS
/*
application constructor
*/
EasymathsApp::EasymathsApp(int* argc, char** argv):
	VDKApplication(argc,argv)
{
}

/*
application destructor
*/
EasymathsApp::~EasymathsApp()
{
}

/*
application setup
*/

void
EasymathsApp::Setup(void)
{
	MainForm = new EasymathsForm(this,NULL);
	MainForm->Setup();
	MainForm->Visible = true;
}

void EasymathsForm::UpdateTitle(bool modified)
{
	char indicator[2];
	if (modified == true) sprintf(indicator, "*");
	if (modified == false) sprintf(indicator," ");
	sprintf(mTitle, "%s [%s%s]", mTITLE_VERSION, basename(mCurrentFile), indicator);
	Title = mTitle;
}

